package net.es.oscars.policy.client;

import org.opensaml.DefaultBootstrap;
import org.opensaml.xml.ConfigurationException;

/**
 * Utility class for bootstrapping OpenSAML. Tracks whether OpenSAML has
 * been bootstrapped previously so it only occurs once. Bootstrapping is
 * required by OpenSAML to do W3C DOM coversions and should be done on 
 * program startup.
 * 
 * @@author Andrew Lake (alake@internet2.edu)
 *
 */
public class SAMLBootstrapper {
    private static boolean isSAMLBootstrapped = false;
    
    /**
     * Bootstraps OpenSAML if it has not yet been bootstrapped. Returns
     * quietly if bootstrapping has previously occurred. 
     * 
     * @throws ConfigurationException
     */
    public static void bootstrap() throws ConfigurationException{
        //Nothing to do if we already bootstrapped SAML
        if(isSAMLBootstrapped){
            return;
        }
        
        DefaultBootstrap.bootstrap();
        isSAMLBootstrapped = true;
    }
}
