package net.es.oscars.policy.ext;

import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;

import org.apache.log4j.Logger;

import net.es.oscars.PropHandler;

public class PolicyExtensionFactory {
    private Logger log;
    private HashMap<String, String> extMap;
    
    public PolicyExtensionFactory(){
        this.log = Logger.getLogger(this.getClass());
        this.extMap = new HashMap<String, String>();
        PropHandler propHandler = new PropHandler("fields.properties");
        Properties props = propHandler.getPropertyGroup("ext", true);
        if(props == null || props.isEmpty()){
            return;
        }
        Enumeration propsEnum = props.keys();
        while(propsEnum.hasMoreElements()){
            String ext = (String)propsEnum.nextElement();
            this.extMap.put(ext, props.getProperty(ext));
        }
    }
    
    public PolicyExtension create(String type) throws RemoteException{
        if(!this.extMap.containsKey(type)){
            throw new RemoteException("Invalid policy extension " + type);
        }
        
        PolicyExtension policyExt = null;
        try {
            policyExt = (PolicyExtension) Class.forName(this.extMap.get(type)).getConstructor().newInstance();
        } catch (Exception e) {
            throw new RemoteException("Unable to load policy extension. Please " +
                    "contact server administrator.");
        }
        
        return policyExt;
    }
}
